function [obj,grad] = ...
    getLR_obj_grad(w,biasD,weights,numRow,...
    penalty,target);

%% returns the value of the objective function (penalized
%% likelihood) and gradient for logistic regression model
%% (actually returns the negative so can use with fminunc

numFeat = length(w)-1;
target = target(:);

%%% LOG LIKELIHOOD
repW = repmat(w,[numRow,1]);
%% get all dot products, z, at once:
z = sum(biasD.*repW,2);
%% feed through the logistic function
zLogit = logistic(z);
tmpLikInd = target.*(zLogit) + ...
    (1-target).*(1-zLogit);
tmpLikIndLog = log(tmpLikInd+eps);
tmpLik = sum(tmpLikIndLog.*weights);
%% NO PENALTY on bias!!! (last column)    
tmpLike = tmpLik - penalty*(w(1:numFeat)*w(1:numFeat)');
obj = -tmpLike;

%% GRADIENT of LOG LIKELIHOOD
%%% original way (verified that this works)
if nargout==2
    %% get difference, and multiply by data vector
    scalarPortion = (target - zLogit); 
    %% weight each part of the derivative
    scalarPortion = scalarPortion.*weights;    
    scalarPortionRep = repmat(scalarPortion,[1,length(w)]);
    deltaW = sum(scalarPortionRep.*biasD,1);
    deltaW = deltaW - [2*penalty*w(1:numFeat) 0];
    grad = -deltaW;
end
if isnan(obj)
    error('here');
end

return;


%% notes
%%
%% ONE data point
%% p(y=1|x) = 1/[1+exp(-wx)] = logistic(wx)
%% p(y=0|x) = 1-p(y=1|x) = 1- 1/[1+exp(-wx)] = 1-logistic(wx)
%%
%% LOG LIKELIHOOD OVER ALL DATA POINTS
%% log_lik = sum_{i=1}^{N} log p(y=label_i|x_i) + ...
%% ... + sum_{j=1}^F w_j^2
%%
%% DERIVATIVE wrt SINGLE WEIGHT, SINGLE DATA POINT
%% d/dw_i [log p(y=1|x)] = 1/p(y=1|x) *...
%%             -1*[1+exp(-wx)]^-2     *...
%%             exp(-wx) * -x
%% d/dw_i [log p(y=0|x)] = 1/p(y=0|x) *...
%%             -1*-1*[1+exp(-wx)]^-2     *...
%%             exp(-wx) * -x
%% d/dw_i [log p(y|x)] = 1/p(y|x) *...
%%             -1*(y==0)*[1+exp(-wx)]^-2     *...
%%             exp(-wx) * x


%%
%% DERIVATIVE wrt SINGLE WEIGHT, ALL DATA POINTS
%% = sum_{i=1}^{N} 



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% new buggy way..
if nargout==2  
	negFlag = -1*(target==0);
    gradScalar = 1./tmpLikInd.*negFlag.*zLogit.^2.*exp(-z);
    gradScalar = gradScalar.*weights;
    %% now multiply by data (biasD)
    repgrad = repmat(gradScalar,[1 length(w)]);        
    gradInd = repgrad.*biasD;
    %% add different cases together weighted appropriately
    grad = sum(gradInd,1);    
    %% NO PENALTY on bias!!! (last column)    
    penaltyTerm = [-2*penalty.*w(1:numFeat) 0];
    grad = grad + penaltyTerm;
    grad = -grad;       
    if any(isnan(grad))
        error('here');
    end
end
