function [f,ind] = accuracy(x1,x2,threshold)
%accuracy - fraction of frames for which the tracking estimate is good
%
% [f,ind] = accuracy(x1,x2,threshold)
% This function computes the distance between each column of x1 and
% the corresponding column of x2, and returns the fraction of these
% distances that are less than 'threshold'.
%    
% Input Arguments:
%   x1, x2 = two equally-sized arrays of, where each column vector 
%            is a point
%   threshold = (default=5)
%
% Return Values:
%   f = fraction of points that are within
%   ind = a binary vector indicating, for each pair of columns, if the
%         distance is less than the threshold.
    
% Author: David Ross, July 2006.
% $Id: accuracy.m,v 1.1 2006/09/12 19:09:36 dross Exp $    
   
error(nargchk(2,3,nargin));

if any(size(x1) ~= size(x2))
    error('sizes of x1 and x2 must match');
end

if nargin < 3
    threshold = 5;
end

ind = sqrt(sum((x1 - x2).^2, 1)) <= threshold;
f = mean(ind);
