function cdf_test(varargin)
% cdf_test(Y,alpha,beta,gamma,eta)
% cdf_test(X,Y,alpha,beta,gamma,eta)

% Author: David Ross
% $Id: cdf_test.m,v 1.6 2006/11/07 18:44:02 dross Exp $

%----------------------------------------------------------
% Check the input
%----------------------------------------------------------
error(nargchk(5,6,nargin));

if length(varargin) == 6
    X = varargin{1};
    varargin(1) = [];
end
    
[Y, alpha, beta, gamma, eta] = varargin{:};
clear varargin;

ground_truth = exist('X','var');

if ground_truth && (numel(X) ~= numel(Y) || ...
    any(cellfun('size',X,2) ~= cellfun('size',Y,2)))
    error('X and Y must have the same sizes');
end            
        

%----------------------------------------------------------
% Run the tests.
%----------------------------------------------------------

N = length(Y);

for nn = 1:N
    %% Do inference
    % full inference, mean-field
    [mu_imf,sigma_imf,pU_imf,pV_imf] = cdf_infer(Y{nn}, ...
        alpha, beta, gamma, eta, 'mean-field');
    if ground_truth
        err_imf = sse(X{nn}([1 4],:) - mu_imf([1 4],:));
        accuracy_imf = accuracy(X{nn}([1 4],:), mu_imf([1 4],:), 5);
        disp(sprintf(['cdf_test: Sequence %d mean-field accuracy %g, ' ...
                      'squared error %g'], nn, accuracy_imf, err_imf));
    end

    % full inference, sampling
    [mu_is,sigma_is,pU_is,pV_is] = cdf_infer(Y{nn}, ...
        alpha, beta, gamma, eta, 'sampling');
    if ground_truth
        err_is = sse(X{nn}([1 4],:) - mu_is([1 4],:));
        accuracy_is = accuracy(X{nn}([1 4],:), mu_is([1 4],:), 5);            
        disp(sprintf(['cdf_test: Sequence %d sampling accuracy %g, ' ...
                      'squared error %g'], nn, accuracy_is, err_is));
    end
    
    % % observations only
    % [mu_obs,sigma_obs] = obs_only(V,Y,beta);
    % if ground_truth
    %     err_obs = sse(X{nn}([1 4],:) - mu_obs([1 4],:))
    % end

    % @TODO add more inference methods here as necessary
    
    %% Plot the results
    figure; hold on;
    legend_strings = {};

    if exist('X','var')
        plot(X{nn}(1,:),X{nn}(4,:),'.k-');
        legend_strings{end+1} = 'ground truth';
    end

    if exist('mu_imf','var')
        plot(mu_imf(1,:),mu_imf(4,:),'.b-');
        legend_strings{end+1} = 'infer mean-field';
    end

    if exist('mu_is','var')
        plot(mu_is(1,:),mu_is(4,:),'.m-');
        legend_strings{end+1} = 'infer sampling';
    end

    if exist('mu_obs','var')
        plot(mu_obs(1,:),mu_obs(4,:),'.r-');
        legend_strings{end+1} = 'observations only';
    end

    if exist('mu_fmf_ref', 'var')
        plot(mu_ref(1,:),mu_ref(4,:),'.g-');
        legend_strings{end+1} = 'filter mean-field refined';
    end

    legend(legend_strings{:},0);
    title(sprintf('cdf_test results, sequence %d', nn), ...
        'Interpreter', 'none');
    hold off;

    %% Save the results
    save(['result' num2str(nn) '.mat']);
end
