function [EF,EG,ER,ES] = feature_expect_given_UV(U,V,Y,alpha,beta,gamma,eta)
% [EF,EG,ER,ES] = feature_expect_given_UV(U,V,Y,alpha,beta,gamma,eta)

% Author: David Ross, 2005
% $Id: feature_expect_given_UV.m,v 1.2 2006/10/04 22:38:02 dross Exp $

%-----------------------------------------------------------
% Check the input
%-----------------------------------------------------------
error(nargchk(7,7,nargin));

% @TODO more checks of input here

%-----------------------------------------------------------
% Initialization
%-----------------------------------------------------------

D = size(alpha,1);
J = size(alpha,3);
K = size(beta,3);
T = size(V,2);

EF = zeros(D,D,J);
EG = zeros(D,D,K);
%ER = zeros(L,J);
%ES = zeros(M,K);

[Trans, dd] = T_matrices;

% new version of Y
rr = [Y(1:end-1).rr];
ss = [Y.ss];
nu = cat(3,Y.nu);
nu = permute(nu, [1 3 2]);


%-----------------------------------------------------------
% Do it.
%-----------------------------------------------------------

[mu,sigma,mu_pair,sigma_pair] = prob_X_given_UVY(Y,U,V,alpha,beta);

% compute E_{X|U,V,Y}[F_j]
for jj = 1:J
    % ind1 and ind2 are indicators which pick out (2:T).*U(jj,:)
    % and (1:T-1).*U(jj,:) respectively.
    ind1 = [false U(jj,:)];
    ind2 = [U(jj,:) false];
    
    if islogical(ind1) & islogical(ind2)
        E_xt_xt = sum(sigma(:,:,ind1),3) ...
            + mu(:,ind1)*mu(:,ind1)';
        E_xt_xt1 = sum(sigma_pair(D+1:end,1:D,U(jj,:)),3) ...
            + mu(:,ind1)*mu(:,ind2)';
        E_xt1_xt1 = sum(sigma(:,:,ind2),3) ...
            + mu(:,ind2)*mu(:,ind2)';
    else
        mu1 = mu .* repmat(ind1, [D 1]);
        mu2 = mu .* repmat(ind2, [D 1]);        
        E_xt_xt = combine(sigma,ind1) + mu1*mu1';
        E_xt_xt1 = combine(sigma_pair(D+1:end,1:D,:), U(jj,:)) + mu1*mu2';
        E_xt1_xt1 = combine(sigma,ind2) + mu2*mu2';
    end

    % this is (mu_t - T_j * mu_{t-1})
    mut_Tmut1 = mu*ind1' - Trans(:,:,jj)*(mu*ind2');

    EF(:,:,jj) = EF(:,:,jj) + E_xt_xt ...
        - E_xt_xt1 * Trans(:,:,jj)' - Trans(:,:,jj) * E_xt_xt1' ...
        + Trans(:,:,jj) * E_xt1_xt1 * Trans(:,:,jj)'  ...
        - dd(:,jj)*mut_Tmut1' - mut_Tmut1*dd(:,jj)' ...
        + (T-1)*dd(:,jj)*dd(:,jj)';
end

% compute E_{X|U,V,Y}[G_k]
if islogical(V)
    for kk = 1:K
        ind = V(kk,:);
        E_xt_xt = sum(sigma(:,:,ind),3) + mu(:,ind)*mu(:,ind)';
        EG(:,:,kk) = EG(:,:,kk) + E_xt_xt ...
            - mu(:,ind)*nu(:,ind,kk)' - nu(:,ind,kk)*mu(:,ind)' ...
            + nu(:,ind,kk)*nu(:,ind,kk)';
    end
else
    for kk = 1:K
        ind = V(kk,:);
        mu1 = mu .* repmat(ind, [D 1]);
        nu1 = nu(:,:,kk) .* repmat(ind, [D 1]);        
        E_xt_xt = combine(sigma,ind) + mu1*mu1';
        EG(:,:,kk) = EG(:,:,kk) + E_xt_xt - mu1*nu1' - nu1*mu1' + nu1*nu1';
    end
end

% Don't forget the -1/2 factor!
EF = -0.5 * EF;
EG = -0.5 * EG;

% Compute E_{X|U,V,Y}[R]
ER = rr * U';

% Compute E_{X|U,V,Y}[S]
ES = ss * V';
