function [EF,EG,ER,ES] = feature_expect_given_X(X,Y,alpha,beta,gamma,eta)
% [EF,EG,ER,ES] = feature_expect_given_X(X,Y,alpha,beta,gamma,eta)

% Author: David Ross, Nov, 2005
% $Id: feature_expect_given_X.m,v 1.2 2006/10/04 22:39:06 dross Exp $

%-----------------------------------------------------------
% Check the input
%-----------------------------------------------------------
error(nargchk(6,6,nargin));

if any(any(any(beta .* (1-repmat(eye(size(beta,1)),[1 1 size(beta,3)])))))
    error(['This will only work for diagonal betas.  ' ...
            'To fix it, change the line marked @DIAGONAL']);
end

%-----------------------------------------------------------
% Initialization
%-----------------------------------------------------------

[Trans,dd] = T_matrices;

% new version of Y
rr = [Y(1:end-1).rr];
ss = [Y.ss];
nu = cat(3,Y.nu);
nu = permute(nu, [1 3 2]);

D = size(alpha,1);
J = size(alpha,3);
K = size(beta,3);
T = size(X,2);
    

%-----------------------------------------------------------
% Do it.
%-----------------------------------------------------------

% First compute the probabilities of U and V given X,Y
[pU,pV] = prob_UV_given_XY(X,Y,alpha,beta,gamma,eta);

EF = zeros(D,D,J);
EG = zeros(D,D,K);
%ER = zeros(L,J);
%ES = zeros(M,K);

% Compute E_{U|X,Y} [F_j]
o = ones(1,T-1);
for jj = 1:J
    f_current = X(:,2:end) - Trans(:,:,jj)*X(:,1:end-1) - dd(:,jj)*o;
    %EF(:,:,jj) = EF(:,:,jj) -0.5 * f_current*diag(pU(jj,:))*f_current';
    EF(:,:,jj) = EF(:,:,jj) -diag(0.5*(f_current.^2)*pU(jj,:)'); %@DIAGONAL
end

% Compute E_{U|X,Y} [G_k]
for kk = 1:K
    g_current = X - nu(:,:,kk);
    %EG(:,:,kk) = EG(:,:,kk) -0.5 *(g_current*diag(pV(kk,:))*g_current');
    EG(:,:,kk) = EG(:,:,kk) -diag(0.5*(g_current.^2)*pV(kk,:)'); % @DIAGONAL
end

% Compute E_{U|X,Y} [R]
ER = rr * pU';

% Compute E_{U|X,Y} [S]
ES = ss * pV';
