function data = load_images(path)
% data = load_images(path)
%
% Read in all of the png image files contained in the directory 'path', and
% return them as a width*height*(# images) array of type uint8, assuming
% 256-level grayscale images.  Images are sorted order, according to 
% Matlab's sort function, based on their filenames.

% Author: David Ross, May 2006.
% $Id: load_images.m,v 1.2 2006/07/18 16:06:19 dross Exp $

EXTENSION = 'png'; % change this to read differed image formats

if path(end) ~= filesep
    path(end+1) = filesep;
end

filenames = dir([path '*.' EXTENSION]);

if length(filenames) == 0
    error(['no image files of type ' EXTENSION ' could be found in ' ...
        path]);
end

filenames = sort({filenames.name});
T = length(filenames);
im = imread([path filenames{1}]);

if ndims(im) > 2
    error(['image ' filenames{1} ' in ' path ' is not black & white']);
end

data = zeros(size(im,1),size(im,2),T,'uint8');
for tt = 1:T
    data(:,:,tt) = imread([path filenames{tt}]);
end
