function [mu,prec] = obs_only(V,Y,beta)
% [mu,prec] = obs_only(V,Y,beta)
% Compute the predictions considering the observations only,
% that is don't use the dynamics features at all.
%
% @TODO this could get better results possibly by using an
% iterative scheme.

% Author: David Ross, Nov. 2005.
% $Id: obs_only.m,v 1.1.1.1 2006/06/01 15:32:22 dross Exp $

%----------------------------------------------------------
% Check the input
%----------------------------------------------------------

error(nargchk(3,3,nargin));

if iscell(Y)
    error('this function only works for one sequence at a time');
end

D = size(beta,1);

nu = compute_g_predictions(Y);
T = size(nu,2);

% the mean and precision of the messages from the observations
msgo_mu = zeros(D,T);
msgo_prec = zeros(D,D,T);


%% Compute the messages from the observations
for tt = 1:T
    % Fast way:
    msgo_prec(:,:,tt) = combine(beta,V(:,tt));
    msgo_mu(:,tt) = combine(mult3(beta,nu(:,tt,:)), V(:,tt));

    % % Slow way
    % for kk = 1:K
    %     msgo_prec(:,:,tt) = msgo_prec(:,:,tt) + beta(:,:,kk)*V(kk,tt);
    %     msgo_mu(:,tt) = msgo_mu(:,tt) + V(kk,tt)*beta(:,:,kk)*nu(:,tt,kk);
    % end

    % msgo_mu(:,tt) = msgo_prec(:,:,tt) \ msgo_mu(:,tt);
    % msgo_mu(:,tt) = pinv(msgo_prec(:,:,tt)) * msgo_mu(:,tt);
    p = diag(msgo_prec(:,:,tt));
    p = (p + realmax*(p==0)).^-1;
    msgo_mu(:,tt) = p .* msgo_mu(:,tt); % @DIAGONAL
end

mu = msgo_mu;
prec = msgo_prec;
