function plot_mu_sigma(mu,sigma,color)
% plot_mu_sigma(mu,sigma[,color])
%
% Plot the means and standard deviations as 1-standard deviation
% ellipses.
%
% Arguments:
%    mu = a 2*T or 6*T matrix of points
%    sigma = a (2*2*T) or (6*6*T) array of covariance matrices
%    color = a color string, like 'r'

% Author: David Ross, Feb. 2005
% $Id: plot_mu_sigma.m,v 1.1.1.1 2006/06/01 15:32:23 dross Exp $

if nargin < 3
    color = 'b';
end

[D,T] = size(mu);
if D == 6
    mu = mu([1 4],:);
    sigma = sigma([1 4],[1 4],:);
end

was_hold =ishold;


plot(mu(1,:),mu(2,:),color);
hold on
for tt = 1:T
    plotEllipse(mu(:,tt),sigma(:,:,tt),color);
end

if ~was_hold
    hold off
end
