function [gamma,eta,U_gt,V_gt] = pretrain_gamma_eta(X,Y)
% [gamma,eta] = pretrain_gamma_eta(X,Y)
% [gamma,eta,U,V] = pretrain_gamma_eta(X,Y)
%
% Use logistic regression to come up with initial estimates of the 
% dynamics and observation switch parameters, gamma and eta.
% This function can also return the (estimated) ground truth U's and V's
% used for training.

% Author: David Ross, Jan. 2005
% $Id: pretrain_gamma_eta.m,v 1.3 2006/10/04 22:41:34 dross Exp $

% initialize the Us and Vs
% @TODO comment this
[Trans,dd] = T_matrices;
J = size(Trans,3);
for nn = 1:length(X)
    nu = cat(3,Y{nn}.nu); 
    nu = permute(nu,[1 3 2]);
    [D,T,K] = size(nu);
    o = ones(1,T-1);
    U{nn} = zeros(J,T-1);
    for jj = 1:J
        err(jj,:) = sum((X{nn}(:,2:end) ...
            - Trans(:,:,jj)*X{nn}(:,1:end-1) - dd(:,jj)*o).^2,1);
    end
    [m,ii] = min(err,[],1);
    U{nn}(ii+(0:J:J*(T-2))) = 1;
    for kk = 1:size(nu,3)
        V{nn}(kk,:) = sqrt(sum((X{nn}([1 4],:) - nu([1 4],:,kk)).^2, 1)) < 15;
    end
end

% save the ground-truth U and V
U_gt = U;
V_gt = V;


%----------------------------------------------------------
% Check the input
%----------------------------------------------------------

error(nargchk(2,4,nargin));

if iscell(V) ~= iscell(Y) 
    error(['U,V and Y must all be cell arrays to handle ' ...
        'multiple sequences']);
end

if iscell(U)
    U_old = U;
    U = [];
    for nn = 1:length(U_old)
        U = [U U_old{nn}];
    end
end

if iscell(V)
    V_old = V;
    V = [];
    for nn = 1:length(V_old)
        V = [V V_old{nn}];
    end
end

if ~iscell(Y)
    Y = {Y};
end

rr = [];
ss = [];
for nn = 1:length(Y)
    T = length(Y{nn});
    rr = [rr Y{nn}(1:end-1).rr];
    ss = [ss Y{nn}.ss];
end

if any(rr(end,:) ~= 1)
    error(['this function assumes the last row of r_features ' ...
        'is an bias always equal to 1, and it''s not true right now']);
end

if any(ss(end,:) ~= 1)
    error(['this function assumes the last row of s_features ' ...
        'is an bias always equal to 1, and it''s not true right now']);
end


%----------------------------------------------------------
% Parameters
%----------------------------------------------------------

penalty = 1;
silent = 1;

J = size(U,1);
K = size(V,1);
L = size(rr,1);
M = size(ss,1);


%----------------------------------------------------------
% Train gamma
%----------------------------------------------------------

gamma = zeros(L,J);

%% Multinomial Case
[junk, labels] = max(U,[],1);
gamma = softmax_regression(rr(1:end-1,:),labels,J);

%% Unigauss case
% for jj = 1:J
%     [w, allLik, allW, allGradSum, it, didConverge] = ...
%         buildLRmodel(rr(1:end-1,:)', U(jj,:), penalty, silent);
%     gamma(:,jj) = w';
% end


%----------------------------------------------------------
% Train eta
%----------------------------------------------------------

% eta = zeros(M,K); % @FULL_ETA
eta = zeros(2,K); % @SPARSE_ETA

for kk = 1:K
    % labels = (V(kk,:) > 0.5) + 1;
    % result = softmax_regression(ss(1:end-1,:),labels);
    % eta(:,kk) = result(:,2);
    
    %% symmetric penalty, each k can look at all of s
    %[w, allLik, allW, allGradSum, it, didConverge] = ... % @FULL_ETA
%    buildLRmodel(ss(1:end-1,:)', V(kk,:), penalty, silent); % @FULL_ETA

    %% symmetric penalty, each k can only look at s(k,:)
    [w, allLik, allW, allGradSum, it, didConverge] = ...   % @SPARSE_ETA
        buildLRmodel(ss(kk,:)', V(kk,:), penalty, silent); % @SPARSE_ETA

    eta(:,kk) = w';
end
