function [pU,pV] = prob_UV_given_XY(X,Y,alpha,beta,gamma,eta)
% [pU,pV] = prob_UV_given_XY(X,Y,alpha,beta,gamma,eta)
% Compute the probabilities of U and V given X and Y.

% Author: David Ross, Oct. 2005
% $Id: prob_UV_given_XY.m,v 1.2 2006/10/04 22:41:34 dross Exp $

%-----------------------------------------------------------
% Check the input.
%-----------------------------------------------------------

error(nargchk(6,6,nargin));

if iscell(X) | iscell(Y)
    error('only works for one sequence at a time');
end

if any(any(any(alpha .* (1-repmat(eye(size(alpha,1)),[1 1 size(alpha,3)])))))
    error(['This will only work for diagonal alphas.  ' ...
            'To fix it, change the line marked @DIAGONAL']);
end
if any(any(any(beta .* (1-repmat(eye(size(beta,1)),[1 1 size(beta,3)])))))
    error(['This will only work for diagonal betas.  ' ...
            'To fix it, change the line marked @DIAGONAL']);
end

% @TODO check the sizes of the input
J = size(alpha,3);
K = size(beta,3);
T = size(Y,2);


%-----------------------------------------------------------
% Do it.
%-----------------------------------------------------------

% new version of Y
rr = [Y(1:end-1).rr];
ss = [Y.ss];
nu = cat(3,Y.nu);
nu = permute(nu, [1 3 2]);
[Trans,dd] = T_matrices;

% ff is a J*(T-1) matrix of f_j(x_{t-1},x_t,Y,alpha_j)
ff = zeros(J,T-1);
o = ones(1,T-1);
for jj = 1:J
    f_current = X(:,2:end) - Trans(:,:,jj)*X(:,1:end-1) - dd(:,jj)*o;
    %ff(jj,:) = -0.5*diag(f_current'*alpha(:,:,jj)*f_current)';
    ff(jj,:) = -0.5*diag(alpha(:,:,jj))'*(f_current.^2); %@DIAGONAL
end

% gg is as K*T matrix of g_k(x_t,Y,beta_k)
gg = zeros(K,T);
for kk = 1:K
    g_current = X - nu(:,:,kk);
    %gg(kk,:) = -0.5*diag(g_current'*beta(:,:,kk)*g_current)';
    gg(kk,:) = -0.5*diag(beta(:,:,kk))'*(g_current.^2); %@DIAGONAL
end

global SOFTMAX_MSG_PRINTED
if isempty(SOFTMAX_MSG_PRINTED)
    disp('using softmax instead of sigmoid for the U''s');
    SOFTMAX_MSG_PRINTED = 1;
end
% pU = sigmoid(ff + gamma'*rr);
pU = softmax(ff + gamma'*rr);
% pV = sigmoid(gg + eta'*ss); % @FULL_ETA
o = ones(1,T); % @SPARSE_ETA
pV = sigmoid(gg + (eta(1,:)'*o).*ss(1:end-1,:) + eta(2,:)'*o); % @SPARSE_ETA
