% run_roll_bounce.m
% Train the tracker on the first 500 frames of the roll and the bounce
% sequences, then test it on the the roll, bounce, and roll+bounce
% sequences.
%
% This script requires the data to be in sub-directories called 
% './mov024', './mov027', and './mov029'.
%
% Trick: Running 'compute_features' takes a long time, but only needs
% to be done once.  During subsequence runs, you can just 
% 'load start.mat', and skip directly to train & test.

% Author: David Ross, 2006
% $Id: run_roll_bounce.m,v 1.2 2007/07/13 19:08:12 dross Exp $

data_paths = {'./mov024', './mov027', './mov029'};
%data_paths = {'/u/dross/datasets/basketball/mov024', ...
%    '/u/dross/datasets/basketball/mov027', ...
%    '/u/dross/datasets/basketball/mov029'};

models = train_features(data_paths{1:2});
save models.mat models;

X = load_points(data_paths{:});
Y = compute_features(models, data_paths{:});

% select the training data
for ii = 1:2
    X_train{ii} = X{ii}(:,1:500);
    Y_train{ii} = Y{ii}(1:500);
end

%% pretrain gamma and eta
[gamma_init, eta_init] = pretrain_gamma_eta(X_train, Y_train);
save start.mat;

%% train
tic;
[alpha,beta,gamma,eta] = cdf_train(X_train, Y_train, [], [], ...
    gamma_init, eta_init);
disp(sprintf('training finished in %g', toc));

%% test
cdf_test(X,Y,alpha,beta,gamma,eta);

%% done
clear X Y X_train Y_train models;
save result.mat;
