% run_simon.m
% Train the tracker on the first 500 frames of the simon sequence,
% then test it on the whole sequence.
%
% This script requires the data to be in a sub-directory called
% './simon'.
%
% Trick: Running 'compute_features' takes a long time, but only needs
% to be done once.  During subsequence runs, you can just 
% 'load start.mat', and skip directly to train & test.

% Author: David Ross, 2006
% $Id: run_simon.m,v 1.3 2007/07/13 19:08:14 dross Exp $

data_paths = {'./simon'};
%data_paths = {'/u/dross/datasets/basketball/simon'};

models = train_features(data_paths{:});
save models.mat models;

X = load_points(data_paths{:});
Y = compute_features(models, data_paths{:});

% select the training data
for ii = 1:numel(X)
    X_train{ii} = X{ii}(:,1:500);
    Y_train{ii} = Y{ii}(1:500);
end

%% pretrain gamma and eta
[gamma_init, eta_init] = pretrain_gamma_eta(X_train, Y_train);
save start.mat;

%% train
tic;
[alpha,beta,gamma,eta] = cdf_train(X_train, Y_train, [], [], ...
    gamma_init, eta_init);
disp(sprintf('training finished in %g', toc));

%% test
cdf_test(X,Y,alpha,beta,gamma,eta);

%% done
clear X Y X_train Y_train models;
save result.mat;
