function y = softmax(y)
% y = softmax(y)
% a numerically stable version of exp(x) / sum(exp(x)),
% performed on each column of y
%
% Author: David Ross

%%%%%
% this snippet, stolen from the NN toolbox, is 
% included for compatibility
% FUNCTION INFO
if isstr(y)
  switch (y)
    case 'deriv', y = '';
    case 'name', y = 'Soft Max';
    case 'output', y = [0 1];
    case 'active', y = [-inf inf];
    case 'type', y = 2;
    otherwise, error('Unrecognized code.')
  end
  return
end
%%%%%


rows = size(y,1);

y = y - repmat(max(y,[],1), [rows 1]) + log(realmax)/2;
y = exp(y);

% normalize along columns
y = y ./ repmat(sum(y,1), [rows 1]);