function [theta] = softmax_regression(X,labels,C)
% theta = softmax_regression(X,labels)
% Train a multiclass linear classification model.
% The posterior class probabilities of a datum x are given by
% softmax(theta'*x).
%
% Required Input:
%    X = data set of column vectors
%    labels = a vector of class labels, one label per column of X
%             labels should be integers in the range 1...#classes
%
% Return value:
%    theta = a matrix of the classification directions, one column
%            vector per class

% Reference: _An Introduction to Probabilistic Graphical Models_ by
% M.I. Jordan, chapter 7 "Linear Classification".

% Author: David Ross, 2003.

error(nargchk(2,3,nargin));

iterations = 800;
learning_rate = 0.01;

X = [X; ones(1,size(X,2))];

if exist('C')
    num_classes = C;
else
    num_classes = max(labels(:));
end
num_examples = size(X,2);

theta = rand(size(X,1), num_classes);
mu = zeros(num_classes, num_examples);

target = zeros(num_classes, num_examples);
for nn = 1:num_examples
    target(labels(nn),nn) = 1;
end

% for it = 1:iterations
%     for nn = 1:num_examples
%         % compute mu
%         mu = theta' * X(:,nn);
%         mu = softmax(mu);
%         
%         % reestimate theta
%         theta = theta + learning_rate * ...
%             X(:,nn) * (target(:,nn) - mu)';
%         
%     end
% end

penalty = 1;
options = optimset('fminunc');
options = optimset(options, 'GradObj','on', ...
    'DerivativeCheck','off', 'Diagnostics','off', ...
    'Display', 'off', 'LargeScale', 'off');
[theta,FVAL,EXITFLAG,OUTPUT] = ...
    fminunc(@err_and_grad, theta, options, target, X, penalty);


%//////////////////////////////////////////////////////////
function [e,g] = err_and_grad(theta,target,X,penalty)

mu = theta'*X;
mu = softmax(mu);

% uncomment this if you don't want the intercept penalized
theta(end,:) = 0;

e = - sum(sum(target .* log(mu))) + penalty/2 * sum(sum(theta.^2));
g = X*(mu-target)' + penalty*theta; 
