% test_prob_X_given_UVY.m
%
% Make sure prob_X_given_UVY is sane by generating a sequence 
% from a known model, then running inference to see if it 
% matches.

% Author: David Ross, July 2006.
% $Id: test_prob_X_given_UVY.m,v 1.1 2006/07/21 20:16:28 dross Exp $

%----------------------------------------------------------
% Create some fake data
%----------------------------------------------------------

K = 5;
T = 500;

[mu0,var0] = get_mu0_var0;
[Trans,dd] = T_matrices;
D = size(Trans,1);
J = size(Trans,3);

U = round(softmax(exp(rand(J,T-1)*10)));
U(1,find(sum(U,1) < 1)) = 1;
V = (rand(K,T) <= 0.5);

alpha = 3*randn(D,D,J);
palpha = zeros(size(alpha));
for jj = 1:J
    alpha(:,:,jj) = alpha(:,:,jj)*alpha(:,:,jj)';
    palpha(:,:,jj) = pinv(alpha(:,:,jj));
end
beta = 3*randn(D,D,K);
pbeta = zeros(size(beta));
for kk = 1:K
    beta(:,:,kk) = beta(:,:,kk)*beta(:,:,kk)';
    pbeta(:,:,kk) = pinv(beta(:,:,kk));
end

X = zeros(D,T);
X(:,1) = mu0+randn(D,1)*1e-2;
for kk = 1:K
    % @TODO if V is off, generate something *really* bad
    Y(1).nu(:,kk) = mvnrnd(X(:,1),pbeta(:,:,kk)+1e10*~V(kk,1))';
end
for tt = 2:T
    jj = find(U(:,tt-1));
    X(:,tt) = Trans(:,:,jj) * X(:,tt-1) + dd(:,jj) + ...
        mvnrnd(zeros(D,1),palpha(:,:,jj))';
    for kk = 1:K
        % @TODO if V is off, generate something *really* bad
        Y(tt).nu(:,kk) = mvnrnd(X(:,tt),pbeta(:,:,kk)+1e10*~V(kk,tt))';
    end
end

%----------------------------------------------------------
% Run inference on the data, see what the result looks like
%----------------------------------------------------------

[mu,sigma,mu_pair,sigma_pair] = prob_X_given_UVY(Y,U,V,alpha,beta);
err = sum(abs(mu-X),1);
plot(err);
disp('these values should be small');
descstat(err)
