function [models] = train_features(varargin)
% [models] = train_features(paths)

%----------------------------------------------------------
% Check the input
%----------------------------------------------------------

error(nargchk(1,Inf,nargin));
paths = varargin;

%----------------------------------------------------------
% Constants
%----------------------------------------------------------

NUM_TRAIN = 500; % @TODO consider moving this magic number
WIDTH = 19;
HEIGHT = 19;

NUM_SEQUENCES = length(paths);

%----------------------------------------------------------
% Train the models
%----------------------------------------------------------

data_small = zeros(HEIGHT, WIDTH, NUM_TRAIN*NUM_SEQUENCES);
    
for ii = 1:NUM_SEQUENCES
    data_path = paths{ii};
    
    % load the true point positions
    load([data_path filesep 'points.mat']);
    if any(points(:) < 0)
        error('points should all be non-negative');
    end
    
    % Load the training images
    data = load_images(data_path);

    %% Restrict it so that it's only a box of SIZE around the true
    %% point
    M =  eye(3);
    M(1,1) = floor(WIDTH/2);
    M(2,2) = floor(HEIGHT/2);
    
    for tt = 1:NUM_TRAIN
        M(1,end) = points(1,tt);
        M(2,end) = points(2,tt);
        window = resample_window_std(double(data(:,:,tt)), M, [WIDTH HEIGHT]);
        data_small(:,:,tt + (ii-1)*NUM_TRAIN) = ...
            reshape(window, HEIGHT, WIDTH);
    end
end

%% construct the models
data_vects = reshape(data_small, HEIGHT*WIDTH, size(data_small,3));
%% Fit the "last image template" model
last_imgs = data_small(:,:,NUM_TRAIN:NUM_TRAIN:end);

if size(last_imgs,3) ~= NUM_SEQUENCES
    error('assert error');
end

%% Fit the Mixture of Gaussians model
K_mog = 5;
[mog_basis, mog_coeffs] = dokmeans(data_vects,K_mog);

%% Fit the PCA model
K_pca = 3;
pca_mean = mean(data_small,3);
[pca_basis, evals] = truepca(data_vects);
pca_basis = pca_basis(:,1:K_pca);

pca_basis_imsize = reshape(pca_basis,[HEIGHT WIDTH K_pca]);

% put it all into a struct
models.pca_mean = pca_mean;
models.pca_basis = pca_basis;
models.template = cat(3, last_imgs, reshape(mog_basis,[HEIGHT WIDTH K_mog]));
